@@ -41,8 +41,8 @@ static ButOut  kill_copyright(ButTimer *timer);
 static bool  color_enabled;
 
 
-void  xio_m_create(Xio *xio, char *name, char *geometry, bool iconic,
-		   bool color)  {
+void  xio_m_create(Xio *xio, const char *name, const char *geometry,
+		   bool iconic, bool color)  {
   int  x, y;
   int  i;
   bd_loc_t  pos;
