@@ -112,7 +112,7 @@ bool  xio_opendisp(Xio *xio)  {
 				xio_shutdown)) == NULL)
     return(FALSE);
   if (first_time)  {
-    char  *lang;
+    const char  *lang;
     int  parses;
 
     first_time = FALSE;
