@@ -1,6 +1,6 @@
 /*
  * wmslib/src/but/box.c, part of wmslib (Library functions)
- * Copyright (C) 1994 William Shubert.
+ * Copyright (C) 1994-1997 William Shubert.
  * See "configure.h.in" for more copyright information.
  */
 
@@ -27,6 +27,23 @@
 #include <but/ctext.h>
 
 
+/**********************************************************************
+ * Data Types
+ **********************************************************************/
+typedef struct  Box_struct  {
+  Pixmap  ulmap, lrmap;
+  int  ulcolor, lrcolor;
+} Box;
+
+typedef struct  BoxFilled_struct  {
+  Pixmap  ulmap, lrmap, cmap;
+  int  ulcolor, lrcolor, ccolor;
+} BoxFilled;
+
+
+/**********************************************************************
+ * Foward Declarations
+ **********************************************************************/
 static void  box_draw(But *but, int x, int y, int w, int h);
 static void  boxFilled_draw(But *but, int x, int y, int w, int h);
 static ButOut  curve(int x, int y, int radius, int start, int sweep,
@@ -40,6 +57,9 @@ static ButOut  box_mmove(But *but, int x, int y);
 #define  CIRCLE  64  /* Points to make a full circle. */
 
 
+/**********************************************************************
+ * Globals
+ **********************************************************************/
 static ButAction  box_action = {
   box_mmove, NULL, NULL, NULL, NULL, NULL,
   box_draw, box_destroy, but_flags, NULL, NULL};
@@ -48,17 +68,10 @@ static ButAction  boxFilled_action = {
   NULL, NULL, NULL, NULL, NULL, NULL,
   boxFilled_draw, boxFilled_destroy, but_flags, NULL, boxFilled_resize};
 
-typedef struct  Box_struct  {
-  Pixmap  ulmap, lrmap;
-  int  ulcolor, lrcolor;
-} Box;
-
-typedef struct  BoxFilled_struct  {
-  Pixmap  ulmap, lrmap, cmap;
-  int  ulcolor, lrcolor, ccolor;
-} BoxFilled;
-
 
+/**********************************************************************
+ * Functions
+ **********************************************************************/
 But  *butBox_create(ButWin *win, int layer, int flags)  {
   But *but;
   Box *box;
@@ -154,10 +167,14 @@ void  but_drawBox(ButWin *win, int x, int y, int w, int h, int bstate,
     points[pnum  ].x = x + h2 - xOff;
     points[pnum++].y = y - yOff;
   } else if (angles & BUT_RLEFT)  {
-    pnum += curve(x+rounding,y+  rounding, rounding-bw,  90,90, points+pnum);
-    pnum += curve(x+rounding,y+h-rounding, rounding-bw, 180,45, points+pnum);
-    pnum += curve(x+rounding,y+h-rounding, rounding, 225,-45, points+pnum);
-    pnum += curve(x+rounding,y+  rounding, rounding, 180,-90, points+pnum);
+    pnum += curve(x+rounding-xOff, y+  rounding-yOff, rounding-bw,
+		  90,90, points+pnum);
+    pnum += curve(x+rounding-xOff, y+h-rounding-yOff, rounding-bw,
+		  180,45, points+pnum);
+    pnum += curve(x+rounding-xOff, y+h-rounding-yOff, rounding,
+		  225,-45, points+pnum);
+    pnum += curve(x+rounding-xOff, y+  rounding-yOff, rounding,
+		  180,-90, points+pnum);
   } else  {  /* Square left edge. */
     points[pnum  ].x = x + bw - xOff;
     points[pnum++].y = y + bw - yOff;
@@ -178,8 +195,10 @@ void  but_drawBox(ButWin *win, int x, int y, int w, int h, int bstate,
     points[pnum  ].x = x + w - (abw + h2) + (abw-bw) - xOff;
     points[pnum++].y = y + bw - yOff;
   } else if (angles & BUT_RRIGHT)  {
-    pnum += curve(x+w-rounding,y+rounding, rounding, 90,-45, points+pnum);
-    pnum += curve(x+w-rounding,y+rounding, rounding-bw,  0,45, points+pnum);
+    pnum += curve(x+w-rounding-xOff, y+rounding-yOff, rounding,
+		  90,-45, points+pnum);
+    pnum += curve(x+w-rounding-xOff, y+rounding-yOff, rounding-bw,
+		  0,45, points+pnum);
   } else  {  /* Square right edge. */
     points[pnum  ].x = x + w - xOff;
     points[pnum++].y = y - yOff;
@@ -222,8 +241,10 @@ void  but_drawBox(ButWin *win, int x, int y, int w, int h, int bstate,
     points[pnum  ].x = x + h2 - xOff;
     points[pnum++].y = y + h - yOff;
   } else if (angles & BUT_RLEFT)  {
-    pnum += curve(x+rounding,y+h-rounding, rounding-bw, 270,-45, points+pnum);
-    pnum += curve(x+rounding,y+h-rounding, rounding, 225,45, points+pnum);
+    pnum += curve(x+rounding-xOff, y+h-rounding-yOff, rounding-bw,
+		  270,-45, points+pnum);
+    pnum += curve(x+rounding-xOff, y+h-rounding-yOff, rounding,
+		  225,45, points+pnum);
   } else  {  /* Square left edge. */
     points[pnum  ].x = x + bw - xOff;
     points[pnum++].y = y + h - bw - yOff;
@@ -240,10 +261,14 @@ void  but_drawBox(ButWin *win, int x, int y, int w, int h, int bstate,
     points[pnum  ].x = x + w - (abw + h2) + (abw-bw) - xOff;
     points[pnum++].y = y + h - bw - yOff;
   } else if (angles & BUT_RRIGHT)  {
-    pnum += curve(x+w-rounding,y+h-rounding, rounding, 270,90, points+pnum);
-    pnum += curve(x+w-rounding,y+rounding, rounding, 0,45, points+pnum);
-    pnum += curve(x+w-rounding,y+rounding, rounding-bw, 45,-45, points+pnum);
-    pnum += curve(x+w-rounding,y+h-rounding, rounding-bw, 0,-90, points+pnum);
+    pnum += curve(x+w-rounding-xOff, y+h-rounding-yOff, rounding,
+		  270,90, points+pnum);
+    pnum += curve(x+w-rounding-xOff, y+rounding-yOff, rounding,
+		  0,45, points+pnum);
+    pnum += curve(x+w-rounding-xOff, y+rounding-yOff, rounding-bw,
+		  45,-45, points+pnum);
+    pnum += curve(x+w-rounding-xOff, y+h-rounding-yOff, rounding-bw,
+		  0,-90, points+pnum);
   } else  {  /* Square right edge. */
     points[pnum  ].x = x + w - xOff;
     points[pnum++].y = y + h - yOff;
@@ -293,7 +318,7 @@ void  but_drawBox(ButWin *win, int x, int y, int w, int h, int bstate,
 static ButOut  curve(int x, int y, int radius, int start, int sweep,
 		     XPoint *pts)  {
   const int  circRadius = 2048;
-  static int  cosine[CIRCLE] = {
+  static const int  cosine[CIRCLE] = {
     2048, 2038, 2009, 1960,   1892, 1806, 1703, 1583,
     1448, 1299, 1138,  965,    784,  595,  400,  201,
     
@@ -359,12 +384,15 @@ void  but_drawCtb(ButWin *win, int flags, int fgpic, int bgpic, int pbgpic,
 		  int x, int y, int w, int h, int bw, int angles)  {
   ButEnv  *env = win->env;
   
-  but_drawBox(win, x, y, w, h, flags & (BUT_PRESSED|BUT_NETPRESS), bw,
+  but_drawBox(win, x, y, w, h,
+	      flags & (BUT_PRESSED|BUT_NETPRESS), bw,
 	      angles, BUT_LIT, BUT_SHAD, None, None);
   if ((flags & (BUT_PRESSED|BUT_NETPRESS)) &&
       (flags & (BUT_TWITCHED|BUT_NETTWITCH|BUT_KEYPRESSED|BUT_NETKEY)))
     bgpic = pbgpic;
   butEnv_setXFg(env, bgpic);
+  x -= win->xOff;
+  y -= win->yOff;
   if (angles)  {
     XPoint  points[CIRCLE + 5];
     int  npoints = 0, abw = (bw * 1448 + 511) / 1024, rnd = env->font0h / 2;
