@@ -1,6 +1,6 @@
 /*
  * wmslib/src/but/but.c, part of wmslib (Library functions)
- * Copyright (C) 1994-1995 William Shubert.
+ * Copyright (C) 1994-1996 William Shubert.
  * See "configure.h.in" for more copyright information.
  */
 
@@ -60,6 +60,10 @@ static Bool  anotherMouseEvent(Display *dpy, XEvent *ev, XPointer arg);
 static int  butErrors(Display *dpy, XErrorEvent *err);
 #endif
 
+
+/**********************************************************************
+ * Globals
+ **********************************************************************/
 ButTimer  *but_timerList;
 Atom  but_wmDeleteWindow, but_wmProtocols;
 
@@ -68,6 +72,10 @@ static struct timeval  long_timeout;
 
 static void  makeStripes(ButEnv *env, int ssize);
 
+/*
+ * This array really should be const, but an error in my Xlib.h forces me
+ *   to leave it non-const.  :-(
+ */
 static char  greymaps[17][4] = {
   {0x00, 0x00, 0x00, 0x00},
   {0x01, 0x00, 0x00, 0x00},
@@ -88,6 +96,9 @@ static char  greymaps[17][4] = {
   {0x0f, 0x0f, 0x0f, 0x0f}};
 
 
+/**********************************************************************
+ * Functions
+ **********************************************************************/
 /* Returns false if the display can't be opened. */
 ButEnv  *butEnv_create(const char *protocol,
 		       const char *dpyname, int shutdown(Display *dpy))  {
@@ -146,7 +157,8 @@ int  butEnv_init(ButEnv *env, void *packet, const char *atomname,
   XVisualInfo  xvi, *vlr;
   int  depth;
   Window  dummyRoot;
-  int  dummyX, dummyY, dummyBWRet, dummyDepth;
+  unsigned int  tempW, tempH, dummyBWRet, dummyDepth;
+  int  dummyX, dummyY;
 
   dpy = env->dpy;
   env->fonts = NULL;
@@ -157,6 +169,7 @@ int  butEnv_init(ButEnv *env, void *packet, const char *atomname,
     but_wmDeleteWindow = XInternAtom(dpy, "WM_DELETE_WINDOW", 0);
     but_wmProtocols = XInternAtom(dpy, "WM_PROTOCOLS", 0);
     depth = DefaultDepth(dpy, DefaultScreen(dpy));
+    env->depth = depth;
   }
   if (firstInit)  {
     firstInit = FALSE;
@@ -181,19 +194,21 @@ int  butEnv_init(ButEnv *env, void *packet, const char *atomname,
   } else
     retval = 0;
   env->packet = packet;
-  env->depth = depth;
   if (dpy)  {
     env->prop = XInternAtom(dpy, atomname, False);
     XGetGeometry(dpy, DefaultRootWindow(dpy), &dummyRoot, &dummyX, &dummyY,
-		 &env->rootW, &env->rootH, &dummyBWRet, &dummyDepth);
+		 &tempW, &tempH, &dummyBWRet, &dummyDepth);
+    env->rootW = tempW;
+    env->rootH = tempH;
   }
   env->sReq = NULL;
   env->sClear = NULL;
   env->sNotify = NULL;
   env->winlist = NULL;
-  env->keepAlive = FALSE;
   env->wllen = env->wlmax = 0;
+  env->minWindows = 1;
   env->butIn = NULL;
+  env->lockBut = NULL;
   env->last_mwin = NULL;
   env->stipDisable = None;
   for (i = 0;  i < BUTWRITE_MAXCHARS;  ++i)
@@ -243,6 +258,10 @@ int  butEnv_init(ButEnv *env, void *packet, const char *atomname,
   env->colorPmaps = (Pixmap *)wms_malloc(env->numColors * sizeof(Pixmap));
   if (!butEnv_stdColors(env))  {
     retval = 1;
+    env->colorp = FALSE;
+    XSetFillStyle(dpy, env->gc, FillTiled);
+    XSetFillStyle(dpy, env->gc2, FillTiled);
+    butEnv_stdColors(env);
   }
   env->partner = 0;
   env->numPartners = 0;
@@ -318,7 +337,7 @@ void  butEnv_events(ButEnv *env)  {
     for (fdGroup = 0;  fdGroup < 3;  ++fdGroup)  {
       fdSets[fdGroup] = env->fMasks[fdGroup];
     }
-    if (env->dpy && (env->wllen == 0) && !env->keepAlive)
+    if (env->dpy && (env->wllen <= env->minWindows))
       return;
     selected_fds = select(env->maxFd, &fdSets[BUT_READFILE],
 			  &fdSets[BUT_WRITEFILE],
@@ -530,6 +549,7 @@ static int  activateEvent(ButEnv *env)  {
     if (!mef.found)  {
       win = butEnv_findWin(env, event.xmotion.window);
       if (win)  {
+	env->eventTime = event.xmotion.time;
 	butWin_mMove(win, event.xmotion.x, event.xmotion.y);
 	butRnet_mMove(env, win->id, event.xmotion.x,event.xmotion.y,
 		      win->w,win->h, -1);
@@ -781,7 +801,7 @@ int  butEnv_setFont(ButEnv *env, int fontnum, const char *fontname,
 		    int fparam)  {
   XFontStruct  *flist;
   char  **fnames;
-  int  i, f_avail;
+  int  i, f_avail, minChar;
   Str  fname, temp;
   int  cstart, fontloaded = 0;
 
@@ -820,6 +840,29 @@ int  butEnv_setFont(ButEnv *env, int fontnum, const char *fontname,
 				&f_avail, &flist);
     if (f_avail > 0)  {
       env->fonts[fontnum] = XLoadQueryFont(env->dpy, fnames[0]);
+      minChar = env->fonts[fontnum]->min_char_or_byte2;
+      if ((minChar > ' ') ||
+	  (env->fonts[fontnum]->max_char_or_byte2 < 'z')) {
+	printf("Char range: %d..%d, should be %d..%d\n",
+	       env->fonts[fontnum]->min_char_or_byte2,
+	       env->fonts[fontnum]->max_char_or_byte2,
+	       ' ', 'z');
+	XFreeFontInfo(fnames, flist, f_avail);
+	continue;
+      }
+      if (env->fonts[fontnum]->per_char == NULL) {
+	if (env->fonts[fontnum]->min_bounds.width < 1) {
+	  XFreeFontInfo(fnames, flist, f_avail);
+	  continue;
+	}
+      } else {
+	if (env->fonts[fontnum]->per_char['a' - minChar].width < 1) {
+	  printf("Width of a is %d\n",
+		 env->fonts[fontnum]->per_char['a'].width);
+	  XFreeFontInfo(fnames, flist, f_avail);
+	  continue;
+	}
+      }
       if (fontnum == 0)  {
 	env->font0h = env->fonts[0]->ascent + env->fonts[0]->descent;
 	env->stdButBw = (env->font0h + 3) / 6;
