@@ -1,6 +1,6 @@
 /*
  * wmslib/src/but/i_general.c, part of wmslib (Library functions)
- * Copyright (C) 1994 William Shubert.
+ * Copyright (C) 1994,1997 William Shubert.
  * See "configure.h.in" for more copyright information.
  */
 
@@ -29,16 +29,21 @@
 #include <but/canvas.h>
 
 
+/**********************************************************************
+ * Forward Declarations
+ **********************************************************************/
 static void  mousein(ButWin *win, int x, int y, ButSet *butset);
 
-typedef struct  {
-  int  x,y,w,h;
-} Rdr;
-
 
+/**********************************************************************
+ * Globals
+ **********************************************************************/
 bool  but_inEvent = FALSE;
 
 
+/**********************************************************************
+ * Functions
+ **********************************************************************/
 void  but_init(But *but)  {
   ButWin  *win = but->win;
   ButEnv  *env = win->env;
@@ -255,8 +260,8 @@ void  but_newFlags(But *but, uint nfl)  {
     env->lockBut = NULL;
   }
   if (nfl != oldflags)  {
-    assert(but->action->newFlags);
-    but->action->newFlags(but, nfl);
+    if (but->action->newFlags)
+      but->action->newFlags(but, nfl);
   }
   if ((oldflags & BUT_DRAWABLE) && !(nfl & BUT_DRAWABLE))  {
     if (but->w > 0)  {
