@@ -32,7 +32,6 @@
  **********************************************************************/
 static ButWin  *create(void *packet, ButEnv *env, const char *name,
 		       int w, int h, ButWinFunc *resize);
-static Window  VirtualRootWindowOfScreen(Screen *screen);
 
 
 /**********************************************************************
@@ -507,55 +506,5 @@ ButColor  butColor_mix(ButColor c1, int r1, ButColor c2, int r2)  {
   return(res);
 }
 
-
-/* From vroot.h by Andreas Stolcke. */
-static Window
-VirtualRootWindowOfScreen(Screen *screen) {
-    static Screen* save_screen = (Screen*) 0;
-    static Window root = (Window) 0;
-
-    if ( screen != save_screen )
-	{
-	Display* dpy = DisplayOfScreen( screen );
-	Atom __SWM_VROOT = None;
-	int i;
-	Window rootReturn, parentReturn;
-	Window* children;
-	unsigned int numChildren;
-
-	root = RootWindowOfScreen( screen );
-
-	/* Go look for a virtual root. */
-	__SWM_VROOT = XInternAtom( dpy, "__SWM_VROOT", False );
-	if ( XQueryTree(
-		 dpy, root, &rootReturn, &parentReturn, &children,
-		 &numChildren ) )
-	    {
-	    for ( i = 0; i < numChildren; ++i)
-		{
-		Atom actual_type;
-		int actual_format;
-		unsigned long nitems, bytesafter;
-		Window* newRoot = (Window*) 0;
-
-		if ( XGetWindowProperty(
-			 dpy, children[i], __SWM_VROOT, 0, 1, False, XA_WINDOW,
-			 &actual_type, &actual_format, &nitems, &bytesafter,
-			 (unsigned char**) &newRoot ) == Success && newRoot )
-		    {
-		    root = *newRoot;
-		    break;
-		    }
-		}
-	    if ( children )
-		XFree( (char*) children );
-	    }
-	save_screen = screen;
-	}
-
-    return root;
-    }
-
-
 #endif  /* X11_DISP */
 
