@@ -1,6 +1,6 @@
 /*
  * wmslib/src/but/net.c, part of wmslib (Library functions)
- * Copyright (C) 1995 William Shubert.
+ * Copyright (C) 1995-1997 William Shubert.
  * See "configure.h.in" for more copyright information.
  */
 
@@ -39,12 +39,15 @@
 #endif
 #include <netdb.h>
 #if  HAVE_ARPA_NAMESER_H
+
 #include <arpa/nameser.h>
 #endif
 #if  HAVE_RESOLV_H
 #include <resolv.h>
 #endif
+#if  HAVE_SYS_UTSNAME_H
 #include <sys/utsname.h>
+#endif
 #include <pwd.h>
 
 #endif  /* HAVE_SOCKETS */
@@ -56,6 +59,9 @@
 
 #if  HAVE_SOCKETS
 
+/**********************************************************************
+ * Forward Declarations
+ **********************************************************************/
 
 static ButOut  newConn(void *packet, int fd);
 static ButRnet  *newRnet(ButEnv *env);
@@ -70,6 +76,10 @@ static void  newFlags(ButRnet *net, int id, uint newFlags);
 static ButOut  butSpec(ButEnv *env, int butId, void *buf, int bufLen);
 
 
+/**********************************************************************
+ * Functions
+ **********************************************************************/
+
 ButLnet  *butLnet_create(ButEnv *env, int port, void *packet,
 			 ButOut ocallback(ButRnet *net),
 			 ButOut callback(ButRnet *net, void *cmd, int cmdLen),
@@ -111,7 +121,7 @@ ButLnet  *butLnet_create(ButEnv *env, int port, void *packet,
 }
 
 
-ButRnet  *butRnet_create(ButEnv *env, char *address, int port,
+ButRnet  *butRnet_create(ButEnv *env, const char *address, int port,
 			 void *packet, ButOut ocallback(ButRnet *net),
 			 ButOut callback(ButRnet *net, void *cmd, int cmdLen),
 			 ButOut ccallback(ButRnet *net, const char *reason))  {
@@ -483,7 +493,9 @@ static void  newFlags(ButRnet *net, int id, uint newFlags)  {
 
 
 static void  putProtoWho(ButRnet *net)  {
+#if  HAVE_SYS_UTSNAME_H
   struct utsname  unameOut;
+#endif
   struct passwd  *pw;
   char  *loginName;
   ButEnv  *env = net->env;
@@ -492,7 +504,9 @@ static void  putProtoWho(ButRnet *net)  {
   assert(MAGIC(net));
   protoOut.length = int_stdInt32(sizeof(protoOut.perType.protoWho));
   protoOut.type = int_stdInt32(BUTNET_PROTOWHO);
+#if  HAVE_SYS_UTSNAME_H
   uname(&unameOut);
+#endif
   if ((loginName = getlogin()) == NULL)  {
     pw = getpwuid(getuid());
     loginName = pw->pw_name;
@@ -501,7 +515,13 @@ static void  putProtoWho(ButRnet *net)  {
   protoOut.perType.protoWho.proto[sizeof(protoOut.perType.protoWho.proto) -
 				  1] = '\0';
   sprintf(protoOut.perType.protoWho.who, "%s@%s",
-	  loginName, unameOut.nodename);
+	  loginName,
+#if  HAVE_SYS_UTSNAME_H
+	  unameOut.nodename
+#else
+	  "unknown"
+#endif
+	  );
   butRnet_write(net, &protoOut);
 }
 
@@ -642,7 +662,7 @@ static ButOut  butSpec(ButEnv *env, int butId, void *buf, int bufLen)  {
 ButLnet  *butLnet_create(ButEnv *env, int port, void *packet,
 			 ButOut ocallback(ButRnet *net),
 			 ButOut callback(ButRnet *net, void *cmd, int cmdLen),
-			 ButOut ccallback(ButRnet *net, char *reason))  {
+			 ButOut ccallback(ButRnet *net, const char *reason))  {
   ButLnet  *net = wms_malloc(sizeof(ButLnet));
 
   net->error = TRUE;
@@ -655,7 +675,7 @@ ButLnet  *butLnet_create(ButEnv *env, int port, void *packet,
 ButRnet  *butRnet_create(ButEnv *env, char *address, int port,
 			 void *packet, ButOut ocallback(ButRnet *net),
 			 ButOut callback(ButRnet *net, void *cmd, int cmdLen),
-			 ButOut ccallback(ButRnet *net, char *reason))  {
+			 ButOut ccallback(ButRnet *net, const char *reason))  {
   ButRnet  *net = wms_malloc(sizeof(ButRnet));
 
   net->error = TRUE;
