@@ -1,6 +1,6 @@
 /*
  * wmslib/src/but/textin.c, part of wmslib (Library functions)
- * Copyright (C) 1994-1996 William Shubert.
+ * Copyright (C) 1994-1997 William Shubert.
  * See "configure.h.in" for more copyright information.
  */
 
@@ -35,6 +35,19 @@
 #include <but/timer.h>
 
 
+/**********************************************************************
+ * Data Types
+ **********************************************************************/
+typedef struct SpecKey_struct {
+  struct SpecKey_struct *next;
+  KeySym keysym;
+  uint keyModifiers, modMask;
+  ButOut (*callback)(But *but, KeySym keysym, uint keyModifiers,
+		     void *context);
+  void *context;
+  MAGIC_STRUCT
+} SpecKey;
+
 typedef struct Txtin_struct  {
   ButOut (*callback)(But *but, const char *value);
   bool  hidden;
@@ -47,15 +60,14 @@ typedef struct Txtin_struct  {
   int  but3StartMouse, but3StartXoff;
   bool  mouseInBut, cursorVisible;
   ButTimer  *cTimer, *sTimer;
+  SpecKey  *specKeys;
   MAGIC_STRUCT
 } Txtin;
 
 
-static int  cutstr_maxlen = 0, cutstr_len;
-static char  *cutdata = NULL;
-static But  *cut_butnum = NULL;
-static But  *paste_butnum = NULL;
-
+/**********************************************************************
+ * Forward Declarations
+ **********************************************************************/
 static int  locateMouse(XFontStruct *fs, char *text, int len, int x,
 			bool *rightSide);
 static void  ti_cursor(But *but);
@@ -84,10 +96,22 @@ static void  wordsel_adjust(Txtin *but);
 static void  startSlide(But  *but, int dir, bool fast);
 static ButOut  slide(ButTimer *timer);
 
-static ButAction  action = {
+
+/**********************************************************************
+ * Globals
+ **********************************************************************/
+static int  cutstr_maxlen = 0, cutstr_len;
+static char  *cutdata = NULL;
+static But  *cut_butnum = NULL;
+static But  *paste_butnum = NULL;
+static const ButAction  action = {
   mmove, mleave, mpress, mrelease,
   kpress, NULL, draw, destroy, flags, NULL};
 
+
+/**********************************************************************
+ * Functions
+ **********************************************************************/
 But  *butTextin_create(ButOut (*callback)(But *but, const char *value),
 		       void *packet, ButWin *win, int layer, int flags,
 		       const char *text, int maxlen)  {
@@ -122,6 +146,7 @@ But  *butTextin_create(ButOut (*callback)(But *but, const char *value),
     ti->cursorVisible = FALSE;
   ti->sTimer = NULL;
   ti->but3StartXoff = -1;
+  ti->specKeys = NULL;
   but_init(but);
   return(but);
 }
@@ -139,9 +164,18 @@ const char  *butTextin_get(But *but)  {
 static ButOut  destroy(But *but)  {
   Txtin  *ti = but->iPacket;
   ButEnv  *env = but->win->env;
+  SpecKey  *spec, *nextSpec;
   
   assert(but->action == &action);
   assert(MAGIC(ti));
+  spec = ti->specKeys;
+  while (spec != NULL) {
+    assert(MAGIC(spec));
+    nextSpec = spec->next;
+    MAGIC_UNSET(spec);
+    wms_free(spec);
+    spec = nextSpec;
+  }
   if (cut_butnum == but)  {
     if (env->sClear == sclear)
       env->sClear = NULL;
@@ -443,9 +477,17 @@ static ButOut  kpress(But *but, const char *newtext, KeySym keysym)  {
   bool  need_draw = FALSE, needCallback = FALSE;
   int  i;
   ButOut  result = 0;
+  SpecKey  *spec;
   
   butTimer_reset(ti->cTimer);
   ti->cursorVisible = TRUE;
+  for (spec = ti->specKeys;  spec != NULL;  spec = spec->next) {
+    if ((keysym == spec->keysym) &&
+	((env->keyModifiers & spec->modMask) == spec->keyModifiers)) {
+      return(spec->callback(but, keysym, env->keyModifiers, spec->context) |
+	     BUTOUT_CAUGHT);
+    }
+  }
   if (((keysym >= XK_KP_Space) && (keysym <= XK_KP_9)) ||
       ((keysym >= XK_space) && (keysym <= XK_ydiaeresis)))  {
     need_draw = TRUE;
@@ -928,8 +970,8 @@ static void  flags(But *but, uint flags)  {
       disableCTimer(but);
     }
   }
-  assert((but->flags & BUT_PRESSABLE) || !(but->flags & BUT_KEYED));
   but->flags = flags;
+  assert((but->flags & BUT_PRESSABLE) || !(but->flags & BUT_KEYED));
   but_draw(but);
 }
 
@@ -1015,7 +1057,7 @@ static void  disableCTimer(But *but)  {
 }
 
 
-void  butTextin_set(But *but, const char *newStr)  {
+void  butTextin_set(But *but, const char *newStr, bool propagate)  {
   Txtin  *ti;
 
   assert(MAGIC(but));
@@ -1027,6 +1069,8 @@ void  butTextin_set(But *but, const char *newStr)  {
   ti->len = ti->loc = ti->cutend = strlen(newStr);
   textInView(but, TRUE);
   but_draw(but);
+  if (propagate)
+    ti->callback(but, ti->str);
 }
 
 
@@ -1052,5 +1096,29 @@ void  butTextin_setHidden(But *but, bool hidden)  {
   }
 }
 
+void  butTextin_setSpecialKey(But *but, KeySym keysym,
+			      uint keyModifiers, uint modMask,
+			      ButOut callback(But *but,
+					      KeySym keysym, uint keyModifiers,
+					      void *context),
+			      void *context) {
+  Txtin  *ti;
+  SpecKey *newSpec;
+
+  assert(MAGIC(but));
+  assert(but->action == &action);
+  ti = but->iPacket;
+  assert(MAGIC(ti));
+  assert((keyModifiers & modMask) == keyModifiers);
+  newSpec = wms_malloc(sizeof(SpecKey));
+  MAGIC_SET(newSpec);
+  newSpec->next = ti->specKeys;
+  ti->specKeys = newSpec;
+  newSpec->keysym = keysym;
+  newSpec->keyModifiers = keyModifiers;
+  newSpec->modMask = modMask;
+  newSpec->callback = callback;
+  newSpec->context = context;
+}
 
 #endif  /* X11_DISP */
